import logging
from shakersynth.synth.rotor import RotorSynth
from shakersynth.config import config
from typing import Dict  # noqa: F401

log = logging.getLogger(__name__)
log.setLevel(config.log_level)


class Aircraft():
    def __init__(self):
        self.synths = [RotorSynth()]
        self.is_running = False

    def update(self, telemetry: dict):
        for synth in self.synths:
            synth.update(telemetry)

    def start(self):
        for synth in self.synths:
            log.debug("Starting synth: %s" % type(synth))
            synth.start()
        self.is_running = True

    def stop(self):
        for synth in self.synths:
            log.debug("Stopping synth: %s" % type(synth))
            synth.stop()
        self.is_running = False



#!/usr/bin/env python3

import logging
import pyo
import sys
from shakersynth.config import config
from shakersynth.aircraft.aircraft import Aircraft
from shakersynth.receiver.shakersynth import ShakersynthReceiver

logging.basicConfig()
log = logging.getLogger("main")
log.setLevel(config.log_level)


def main():
    print("\n" + "-" * 35 + "Shakersynth" + "-" * 35)
    # Audio synthesis setup.
    api = config.audio_api
    print(f"Using audio API: {api}")
    server = pyo.Server(
        nchnls=2,
        duplex=0,
        buffersize=config.buffer_size,
        winhost=api,
        sr=config.sample_rate
    )
    server.setVerbosity(1)

    # Map audio outputs from internal numbers (which can be sparse and large)
    # to a friendly, 1-indexed, monotonic series for the user.
    output_devices = pyo.pa_get_devices_infos()[1]
    friendly_to_internal = {}

    print("Found these audio outputs:")
    device_details = enumerate(output_devices.items())
    for friendly_index, (internal_index, properties) in device_details:
        # Only list devices that are available using the default audio API.
        if properties["host api index"] == pyo.pa_get_default_host_api():
            friendly_index += 1
            friendly_to_internal[friendly_index] = internal_index
            print(f"  {friendly_index}: {properties['name']}")

    print("Enter device ID to use: ", end="")
    chosen_device = int(input())
    server.setOutputDevice(friendly_to_internal[chosen_device])

    server.boot()
    server.start()
    server.setAmp(config.global_volume)

    # Receive Telemetry from the Shakersynth DCS export script.
    receiver = ShakersynthReceiver()

    # All telemetry processing and sound synthesis happens in the aircraft
    # model.
    aircraft = Aircraft()

    print("*" * 80)
    print("ShakerSynth now running.")
    print("Press ctrl+c to exit.")
    print("*" * 80)
    log.info("Waiting for initial telemetry.")

    # A little state machine to keep track of our aircraft and telemetry.
    while True:
        telemetry = receiver.get_telemetry()
        running = aircraft.is_running

        if telemetry and not running:
            # Then we have just started a mission.
            log.info("Starting new aircraft: %s" % telemetry["module"])
            aircraft.update(telemetry)
            aircraft.start()

        elif telemetry and running:
            # Then things are ticking along nicely.
            aircraft.update(telemetry)

        elif running and not telemetry:
            # Then we got an empty telemetry object while running an aircraft.
            # This signifies that we left the mission or lost contact with DCS.
            log.info("Shutting down aircraft.")
            aircraft.stop()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(0)



import socket


class SimShakerReceiver():
    def __init__(self, port=29377):
        if port is not None:
            self.listener = socket.socket(type=socket.SOCK_DGRAM)
            self.listener.bind(('', port))

    def get_telemetry(self):
        """Return the next available telemetry payload.

        Blocks until one is available.
        """
        message = self.listener.recv(1500)
        simshaker = self.parse(message.decode())

        telemetry = {}

        # Module name
        try:
            module = simshaker["N"].lower()
        except KeyError:
            # Not a telemetry packet from SimShaker.
            # Probably a "start" or "stop" message.
            return {}
        if module == "mi-8mt":
            telemetry["module"] = "mi-8"
        else:
            telemetry["module"] = module

        # Rotor RPM
        try:
            rpm_percent = float(simshaker["RotorRPM"])
        except KeyError:
            rpm_percent = 0.0
        telemetry["rotor_rpm_percent"] = rpm_percent

        return telemetry

    def parse(self, message):
        """Parse a SimShaker telemetry message into a dict."""
        simshaker_telemetry = {}
        elements = message.split(';')

        for element in elements:
            try:
                key, value = element.split("=")
                simshaker_telemetry[key] = value
            except ValueError:
                pass

        return simshaker_telemetry



import logging
import socket
import yaml
from func_timeout import func_set_timeout, FunctionTimedOut  # type: ignore
from shakersynth.config import config

log = logging.getLogger(__name__)
log.setLevel(config.log_level)


class ShakersynthReceiver():
    """Recieve YAML encoded telemetry from Shakersynth's DCS export script."""

    def __init__(self, bind_addr="", port=17707):
        """Create a telemetry receiver listening on a UDP socket."""
        self.packet_count = 0
        if port is not None:
            self.listener = socket.socket(type=socket.SOCK_DGRAM)
            self.listener.bind((bind_addr, port))

    @func_set_timeout(1)
    def recieve_udp(self):
        """Recieve a UDP packet.

        Raises FunctionTimedOut if one is not received within 1 second.
        """
        return self.listener.recv(1500)

    def get_telemetry(self):
        """Return the next available telemetry payload.

        Returns an empty dictionary if nothing is received within 1 second.
        """
        try:
            payload = self.recieve_udp()
            self.packet_count += 1
        except FunctionTimedOut:
            log.debug('No telemetry...')
            return {}

        # YAML may seem like a strange wire format, but it's very fast and
        # clean to hand-craft it in the export script. Decoding it may be
        # relatively expensive, but we have cycles to spare here in the Python
        # world.
        telemetry = yaml.safe_load(payload.decode())

        # Capital letters hurt your hands.
        try:
            module = telemetry["module"].lower()
        except KeyError:
            # No module is active. Return an empty telemetry object signifing
            # that we not currently in an aircraft.
            return {}

        if module == "mi-8mt":  # Nobody says "Mi-8MT".
            module = "mi-8"     # We just say "Mi-8".

        telemetry["module"] = module

        if self.packet_count % 60 == 0:
            log.debug("Telemetry sample: %s" % telemetry)
        return telemetry



import logging
import os
import platform
import sys
import yaml

from textwrap import dedent

#try:
log_level = getattr(
    logging,
    os.getenv('SHAKERSYNTH_LOG_LEVEL', 'info').upper()
)
#except KeyError:
#    log_level = logging.INFO

default_yaml = dedent(
    """
    # Select and configure the audio API.
    # Options on Windows are "mme", "directsound", "asio", "wasapi", "wdm-ks".
    #
    # See also: http://ajaxsoundstudio.com/pyodoc/winaudioinspect.html
    audio_api: wasapi
    sample_rate: 44100
    buffer_size: 1024

    # Set the overall volume between 0 and 1.0.
    global_volume: 0.90
    """
    ).strip()

if platform.system() == 'Windows':
    config_dir = os.path.join(
        os.environ['LOCALAPPDATA'],
        'Shakersynth'
    )
else:
    config_dir = os.path.join(
        os.environ['HOME'],
        '.shakersynth'
    )
config_file_path = os.path.join(config_dir, 'shakersynth.yml')

if not os.path.exists(config_dir):
    os.mkdir(config_dir)

if not os.path.exists(config_file_path):
    with open(config_file_path, 'w') as config_file:
        config_file.write(default_yaml)

config_yaml = open(config_file_path).read()
config_from_file = yaml.safe_load(config_yaml)

# Map all options in the config file to attributes of this "config" module.
config_module = sys.modules[__name__]
for key, value in config_from_file.items():
    setattr(config_module, key, config_from_file[key])